#include "SceneTemplate.h"
#include "log.h"
#include "string.h"
#include "malloc.h"


/**
 * ======== SceneTemplate::SceneTemplate ========
 *
 *  Purpose:
 *    Scene templates are VRML templates that get nodes substituted into them.
 *    This is to allow flexibility in how the shapes get treated, and to allow
 *    some level of programming to happen with the VRML.  This would allow the
 *    functionality of the renderer to changed based only on the external text.
 *
 *  Input Parameters:
 *    name -- the name of the scene template
 */

SceneTemplate::SceneTemplate( char* name ) {
	templateName = newstrcpy( name );
	sceneTemplateLength = 0;
	templateChars = newstrcpy( "" );
}


/**
 * ======== SceneTemplate::addLine ========
 *
 *  Purpose:
 *    Add a line to the scene template.  Expand the memory if necessary.
 *
 *  Input Parameters:
 *    line -- the line of text to add to the template.
 */

void SceneTemplate::addLine( char* line ) {
	unsigned int requiredLength = ((strlen( line ) + 2 + strlen( templateChars ) + 1)*100)/100;
	if ( requiredLength != sceneTemplateLength ) {
		sceneTemplateLength = requiredLength;
		char* newTemplate = (char*)malloc( sceneTemplateLength );
		strcpy( newTemplate, templateChars );
		strcat( newTemplate, line );
		free( templateChars );
		templateChars = newTemplate;
	} else {
		strcat( templateChars, line );
	}
	strcat( templateChars, "\n" );
}



/**
 *  ======== SceneTemplateManager::SceneTemplateManager ========
 *
 *  Purpose:
 *    Manage a list of SceneTemplate objects.  Since I don't expect alot of
 *    these to exist, hard coded fixed limit of 200 (see header file).
 */

SceneTemplateManager::SceneTemplateManager() {
	templateIdx = 0;
}


/**
 * ======== SceneTemplateManager::getTemplateChars ========
 *
 *  Purpose:
 *    Look up the template characters by name
 *
 *  Input Parameters:
 *    name -- the name of the SceneTemplate to find
 *
 *  Return Values:
 *    The characters of the template, or NULL if the template is not found.
 */

char* SceneTemplateManager::getTemplateChars( char* name ) {
	for ( int i = 0; i < templateIdx; i++ ) {
		SceneTemplate* test = templateList[i];
		if ( strcmp( name, test->getName() ) == 0 ) {
			return( test->getChars() );
		}
	}
	return( NULL );
}


/**
 * ======== SceneTemplateManager::addTemplate ========
 *
 *  Purpose:
 *    Create a new SceneTemplate and add it to the list.
 *
 *  Input Parameters:
 *    name -- the name of the template.  All scene templates are accessed by name
 */

SceneTemplate* SceneTemplateManager::addTemplate( char* name ) {
	//log_message1( "SceneTemplateManager::addTemplate", name );
	SceneTemplate* s = new SceneTemplate( name );
	templateList[ templateIdx++ ] = s;
	return( s );
}


/**
 * ======== SceneTemplateManager::addLine ========
 *
 *  Purpose:
 *    Add a line of text to the most recently added SceneTemplate
 *
 *  Input Parameters:
 *    line -- the line of text to add
 */

void SceneTemplateManager::addLine( char* line ) {
	//log_message1( "SceneTemplateManager::addLine", line );
	SceneTemplate* latest = templateList[ templateIdx - 1 ];
	//log_message1( "latest template name is: ", latest->getName() );
	latest->addLine( line );
}
